@@ -2,14 +2,32 @@
 #' (<https://www.jamovi.org>)
 #'
 #' @param dtaFrm Data frame to be exported (default: NULL)
-#' @param fleOut Name / position of the output file to be generated ("FILENAME.omv"; default: "")
-#' @param retDbg Whether to return a list with debugging information (see Value; default: FALSE)
+#' @param fleOut Name / position of the output file to be generated
+#'               ("FILENAME.omv"; default: "")
+#' @param wrtPtB Whether to write protocol buffers (see Details; default: FALSE)
+#' @param frcWrt Whether to overwrite existing files with the same name (see Details; default: FALSE)
+#' @param retDbg Whether to return a list with debugging information (see
+#'               Value; default: FALSE)
 #'
-#' @return a list (if retDbg == TRUE), containing the meta data (mtaDta, metadata.json in the OMV-file), the extended data (xtdDta, xdata.json in the OMV-file) and the original data frame (dtaFrm)
+#' @return a list (if retDbg == TRUE), containing the meta data (mtaDta,
+#'         metadata.json in the OMV-file), the extended data (xtdDta,
+#'         xdata.json in the OMV-file) and the original data frame (dtaFrm)
 #'
 #' @details
-#' jamovi has a specific measurement level / type "ID" (in addition to the "standard" ones "Nominal", "Ordinal", and "Continuous"). "ID" is used for columns that contain some form of ID (e.g., a
-#' participant code). In order to set a variable of your data frame to "ID", you have to manually set an attribute \code{jmv-id} (e.g., \code{attr(dtaFrm$column, "jmv-id") = TRUE}).
+#' * jamovi has a specific measurement level / type "ID" (in addition to the
+#'   "standard" ones "Nominal", "Ordinal", and "Continuous"). "ID" is used for
+#'   columns that contain some form of ID (e.g., a participant code). In order
+#'   to set a variable of your data frame to "ID", you have to set the
+#'   attribute `jmv-id` (e.g., `attr(dtaFrm$column, "jmv-id") = TRUE`).
+#' * CAUTION: Setting wrtPtB to TRUE currently overwrites analyses that already
+#'   exist in a data file. It is meant to be used for `describe_omv` only. If
+#'   you set wrtPtB to TRUE, ensure to use an output file name that isn't would
+#'   not overwrite any existing file. Protocol buffers are used to exchange
+#'   data between the different parts of jamovi (the server and the client) and
+#'   also the format in which analyses are stored in the jamovi data files.
+#' * `write_omv` checks whether the output file already exists and throws an
+#'   error if this is the case. frcWrt permits you to overwrite the existing
+#'   file.
 #'
 #' @examples
 #' \dontrun{
@@ -18,10 +36,10 @@
 #' # use the data set "ToothGrowth" and, if it exists, write it as
 #' # jamovi-file using write_omv()
 #' data("ToothGrowth")
-#' fleOMV <- paste0(tempfile(), ".omv")
+#' nmeOut <- tempfile(fileext = ".omv")
 #' # typically, one would use a "real" file name instead of tempfile(),
 #' # e.g., "Data1.omv"
-#' dtaDbg = write_omv(ToothGrowth, fleOMV, retDbg = TRUE)
+#' dtaDbg = write_omv(dtaFrm = ToothGrowth, fleOut = nmeOut, retDbg = TRUE)
 #' print(names(dtaDbg))
 #' # the print-function is only used to force devtools::run_examples()
 #' # to show output
@@ -33,16 +51,16 @@
 #'
 #' # check whether the file was written to the disk, get the file informa-
 #' # tion (size, etc.) and delete the file afterwards
-#' print(list.files(dirname(fleOMV), basename(fleOMV)))
+#' print(list.files(dirname(nmeOut), basename(nmeOut)))
 #' # -> "file[...].omv" ([...] is a combination of random numbers / characters
-#' print(file.info(fleOMV)$size)
-#' # -> approx. 2500 (size may differ on different OSes)
-#' unlink(fleOMV)
+#' print(file.info(nmeOut)$size)
+#' # -> approx. 2600 (size may differ on different OSes)
+#' unlink(nmeOut)
 #' }
 #'
 #' @export write_omv
 
-write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
+write_omv <- function(dtaFrm = NULL, fleOut = "", wrtPtB = FALSE, frcWrt = FALSE, retDbg = FALSE) {
     if (is.null(dtaFrm)) stop("The data frame to be written needs to be given as parameter (dtaFrm = ...).")
     if (!nzchar(fleOut)) stop("Output file name needs to be given as parameter (fleOut = ...).")
 
@@ -50,9 +68,18 @@ write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
     fleOut <- nrmFle(fleOut)
     chkDir(fleOut)
     chkExt(fleOut, "omv")
+    if (file.exists(fleOut)) {
+        if (frcWrt) {
+            unlink(fleOut)
+        } else {
+            stop("The output file already exists. Either remove the file or set the parameter frcWrt to TRUE.")
+        }
+    }
 
     # check whether dtaFrm is a data frame
+    # attach dataType and measureType attributes when inside jamovi
     chkDtF(dtaFrm)
+    if (isJmv()) dtaFrm <- jmvAtt(dtaFrm)
 
     # handle the attributes "variable.labels" and "value.labels" in the format provided by the R-package "foreign"
     # the attribute "variable.labels" (attached to the data frame) is converted them to the format used by jamovi ("jmv-desc" attached to the data column)
@@ -102,13 +129,14 @@ write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
         crrCol <- dtaFrm[[i]]
 
         # ID variables represent a special case and are therefore treated first
-        # if the jmv-id marker is set or if the measureType is set to "ID" in the original data or if it is the first column with the values
-        # being unique and being either a factor or an integer (rounded equals the original value; integers may be stored as doubles)
-        if (chkAtt(dtaFrm[[i]], "jmv-id", TRUE) || chkAtt(dtaFrm[[i]], "measureType", "ID") ||
-          (i == 1 && length(unique(crrCol)) == length(crrCol)) && (is.factor(crrCol) || (is.double(crrCol) && all(crrCol %% 1 == 0)))) {
+        # if the jmv-id marker is set or if the measureType is set to "ID" in the original data, or if it is the first column that hasn't the attribute
+        # measureType attached, has values that are unique and not NA and is either a factor or an integer (rounded equals the original value)
+        if (chkAtt(crrCol, "jmv-id", TRUE) || chkAtt(crrCol, "measureType", "ID") ||
+             (i == 1 && !chkAtt(crrCol, "measureType") && !any(duplicated(crrCol) | is.na(crrCol)) &&
+               (is.character(crrCol) || is.factor(crrCol) || (is.numeric(crrCol) && all(crrCol %% 1 == 0))))) {
             mtaDta$fields[[i]][["measureType"]] <- "ID"
-            mtaDta$fields[[i]][["dataType"]]    <- ifelse(is.integer(crrCol), "Integer", "Text")
-            mtaDta$fields[[i]][["type"]]        <- ifelse(is.integer(crrCol), "integer", "string")
+            mtaDta$fields[[i]][["dataType"]]    <- ifelse(is.numeric(crrCol), "Integer", "Text")
+            mtaDta$fields[[i]][["type"]]        <- ifelse(is.numeric(crrCol), "integer", "string")
         # afterwards, the different variable types for each column of the original data frame are tested
         # an overview about how jamovi treats variable types internally and as which types they are written
         # can be found in the function jmvAtt under globals.R
@@ -135,7 +163,7 @@ write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
             # above must be kept at crrCol as the original column might be character and was converted above
             facOrd <- is.ordered(dtaFrm[[i]])
             if (chkAtt(dtaFrm[[i]], "values") && !identical(attr(dtaFrm[[i]], "values"), as.integer(attr(dtaFrm[[i]], "levels")))) {
-                clsRmv(list(binHdl, strHdl))
+                clsRmv()
                 stop(sprintf(paste("\"values\"-attribute with unexpected values found for column \"%s\".",
                                    "Please send the file to sebastian.jentschke@uib.no for debugging."), names(dtaFrm[i])))
             }
@@ -191,23 +219,24 @@ write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
             mtaDta$fields[[i]][["description"]] <- paste(c(ifelse(nzchar(mtaDta$fields[[i]][["description"]]), mtaDta$fields[[i]][["description"]], names(dtaFrm[i])),
                                                          "(time converted to numeric; sec since 00:00)"), collapse = " ")
         } else {
-            clsRmv(list(binHdl, strHdl))
+            clsRmv()
             stop(sprintf("Variable type %s not implemented. Please send the data file that caused this problem to sebastian.jentschke@uib.no", class(crrCol)))
         }
 
         # remove atrributes that are only used with specific columnTypes
-        if (all(class(dtaFrm[[i]]) != c("logical", "factor"))) {
+        if (chkFld(mtaDta$fields[[i]], "columnType", "Filter")) {
+            # if the variable is a filter, trimLevels is removed in any case
             mtaDta$fields[[i]][["trimLevels"]]               <- NULL
-        }
-        if (! chkFld(mtaDta$fields[[i]], "columnType", "Filter")) {
+        } else {
+            # if the variable isn't a filter, trimLevels is only removed
+            # if the original variable was a factor / logical
+            if (all(class(dtaFrm[[i]]) != c("logical", "factor"))) {
+                mtaDta$fields[[i]][["trimLevels"]]           <- NULL
+            }
             mtaDta$fields[[i]][["filterNo"]]                 <- NULL
             mtaDta$fields[[i]][["active"]]                   <- NULL
-        } else {
-            # if the variable is a filter, trimLevels is removed even in cases
-            # where the original variable was a factor / logical
-            mtaDta$fields[[i]][["trimLevels"]]               <- NULL
         }
-        if (! chkFld(mtaDta$fields[[i]], "columnType", "Output")) {
+        if (!chkFld(mtaDta$fields[[i]], "columnType", "Output")) {
             mtaDta$fields[[i]][["outputAnalysisId"]]         <- NULL
             mtaDta$fields[[i]][["outputOptionName"]]         <- NULL
             mtaDta$fields[[i]][["outputName"]]               <- NULL
@@ -226,6 +255,7 @@ write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
         } else if (chkFld(mtaDta$fields[[i]], "type", "number"))  {
             wrtCol <- as.double(crrCol)
         } else if (chkFld(mtaDta$fields[[i]], "type", "string"))  {
+            if (!is.character(crrCol)) crrCol <- as.character(crrCol)
             crrCol[is.na(crrCol)] <- ""
             wrtCol <- rep(NA, length(crrCol))
             for (j in seq_along(crrCol)) {
@@ -235,9 +265,6 @@ write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
                 strPos <- strPos + length(charToRaw(crrCol[j])) + 1
             }
             wrtCol <- as.integer(wrtCol)
-        } else {
-            clsRmv(list(binHdl, strHdl))
-            stop(sprintf("Variable type %s not implemented. Please send the data file that caused this problem to sebastian.jentschke@uib.no", mtaDta$fields[[i]][["type"]]))
         }
         writeBin(wrtCol, binHdl, endian = "little")
 
@@ -254,39 +281,38 @@ write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
     }
 
     # compress data.bin and discard the temporary file
-    add2ZIP(fleOut, binHdl, newFle = TRUE)
-    rm(binHdl)
+    add2ZIP(fleOut, crrHdl = binHdl)
 
     # compress strings.bin (only if it contains data) and discard the temporary file
-    add2ZIP(fleOut, strHdl, blnZIP = (strPos > 0))
-    rm(strHdl, strPos)
+    add2ZIP(fleOut, crrHdl = strHdl, incZIP = (strPos > 0))
+    rm(strPos)
 
     # create meta, write it and add it to ZIP file
-    mnfHdl <- file(file.path(tempdir(), "meta"),         open = "wb")
-    add2ZIP(fleOut, mnfHdl, txtOut = mnfTxt())
-    rm(mnfHdl)
+    add2ZIP(fleOut, crrFle = c("meta", "wb"), txtOut = mnfTxt())
 
     # write metadata.json
-    mtaHdl <- file(file.path(tempdir(), "metadata.json"), open = "w")
-    add2ZIP(fleOut, mtaHdl, txtOut = fmtJSON(list(dataSet = mtaDta)))
-    rm(mtaHdl)
+    add2ZIP(fleOut, crrFle = "metadata.json", txtOut = fmtJSON(list(dataSet = mtaDta)))
 
     # write xdata.json
-    xtdHdl <- file(file.path(tempdir(), "xdata.json"),    open = "w")
-    add2ZIP(fleOut, xtdHdl, txtOut = fmtJSON(xtdDta))
-    rm(xtdHdl)
+    add2ZIP(fleOut, crrFle = "xdata.json",    txtOut = fmtJSON(xtdDta))
 
-    # write index.html and add it to ZIP file
-    # currently, the HTML that is stored in the HTML attribute can't be saved because it is only a "front"
-    # that doesn't work without the analyses and images contained in it being stored too
-    htmHdl <- file(file.path(tempdir(), "index.html"),    open = "w")
-#   if (!is.null(attr(dtaFrm, "HTML"))) {
-#      add2ZIP(fleOut, htmHdl, txtOut = attr(dtaFrm, "HTML"))
-#   } else {
-#      add2ZIP(fleOut, htmHdl, txtOut = htmTxt())
-#   }
-    add2ZIP(fleOut, htmHdl, txtOut = htmTxt())
-    rm(htmHdl)
+    # if "HTML" attribute exists (e.g., because functions such as describe_omv
+    # did modify it), this attribute is written; otherwise, htmTxt (a clean
+    # "index.html", i.e., without any results) is written
+    if (!is.null(attr(dtaFrm, "HTML"))) {
+        add2ZIP(fleOut, crrFle = "index.html", txtOut = attr(dtaFrm, "HTML"))
+    } else {
+        add2ZIP(fleOut, crrFle = "index.html", txtOut = htmTxt())
+    }
+
+    if (wrtPtB) {
+        if (is.null(attr(dtaFrm, "protobuf")) || length(attr(dtaFrm, "protobuf")) < 1 || !inherits(attr(dtaFrm, "protobuf")[[1]], "Message")) {
+            unlink(fleOut)
+            stop("The data frame (dtaFrm) must contain the attribute \"protobuf\", there has to be at least one of them, and it has to be of the correct type (a RProtoBuf).")
+        }
+        # write ProtoBuffers
+        add2ZIP(fleOut, crrFle = c(names(attr(dtaFrm, "protobuf")[1]), "wb"), ptbOut = attr(dtaFrm, "protobuf")[[1]])
+    }
 
     # handle weights
     if (chkAtt(dtaFrm, "weights", "\\w+")) {
@@ -299,64 +325,69 @@ write_omv <- function(dtaFrm = NULL, fleOut = "", retDbg = FALSE) {
     }
 }
 
-clsRmv <- function(fleHdl = NULL) {
-    for (crrHdl in fleHdl) {
-        crrFle <- summary(crrHdl)[["description"]]
-        close(crrHdl)
-        unlink(crrFle)
-        rm(crrFle)
-    }
-}
-
+# convert to JSON and do some beatifying (adding spaces for increased legibility)
 fmtJSON <- function(txtJSON = "") {
     gsub("\"weights\": \\{\\}", "\"weights\": null", gsub("00: 00", "00:00", gsub("  ", " ", gsub(":", ": ", gsub(",", ", ",
       jsonlite::toJSON(txtJSON, auto_unbox = TRUE))))))
 }
 
+# generates an “empty” index.html (i.e., an index.html not containing any results output)
 htmTxt <- function() {
     c("<!DOCTYPE html>",
       "<html>",
-      "    <head>",
-      "        <meta charset=\"utf-8\" />",
-      "        <title>Results</title>",
-      "        <style>\n",
-      "            body {",
-      "                font-family: \"Segoe UI\",Roboto,Helvetica,Arial,sans-serif,\"Segoe UI Emoji\",\"Segoe UI Symbol\" ;",
-      "                font-size: 12px ;",
-      "                color: #333333 ;",
-      "                margin: 24px;",
-      "                cursor: default ;",
-      "            }\n",
-      "            h1 {",
-      "                font-size: 160% ;",
-      "                color: #3E6DA9 ;",
-      "                margin-bottom: 12px ;",
-      "                white-space: nowrap ;",
-      "            }\n",
-      "            h2 {",
-      "                font-size: 130% ;",
-      "                color: #3E6DA9 ;",
-      "                margin-bottom: 12px ;",
-      "            }\n",
-      "            h3, h4, h5 {",
-      "                font-size: 110% ;",
-      "                margin-bottom: 12px ;",
-      "            }\n",
-      "            table {",
-      "                page-break-inside: avoid;",
-      "                border-spacing: 0 ;",
-      "            }\n",
-      "            table tr td, table tr th {",
-      "                font-size: 12px ;",
-      "                page-break-inside: avoid;",
-      "            }\n",
-      "        </style>",
+      "<head>",
+      "    <meta charset=\"utf-8\" />",
+      "    <title>Results</title>",
+      "    <style>\n",
+      "    body {",
+      "        font-family: \"Segoe UI\",Roboto,Helvetica,Arial,sans-serif,\"Segoe UI Emoji\",\"Segoe UI Symbol\" ;",
+      "        color: #333333 ;",
+      "        cursor: default ;",
+      "        margin: 24px;",
+      "        font-size: 12px ;",
+      "    }\n",
+      "    h1 {",
+      "        font-size: 160% ;",
+      "        color: #3E6DA9 ;",
+      "        margin-bottom: 12px ;",
+      "        white-space: nowrap ;",
+      "    }\n",
+      "    h2 {",
+      "        font-size: 130% ;",
+      "        margin-bottom: 12px ;",
+      "        color: #3E6DA9 ;",
+      "    }\n",
+      "    h3, h4, h5 {",
+      "        font-size: 110% ;",
+      "        margin-bottom: 12px ;",
+      "    }\n",
+      "    table {",
+      "        border-spacing: 0 ;",
+      "        page-break-inside: avoid;",
+      "    }\n",
+      "    table tr td, table tr th {",
+      "        page-break-inside: avoid;",
+      "        font-size: 12px ;",
+      "    }\n",
+      "    .ql-align-center {\n        text-align: center;\n    }\n",
+      "    .ql-align-right {\n        text-align: right;\n    }\n",
+      "    .ql-align-justify {\n        text-align: justify;\n    }\n",
+      "    .ql-indent-1 {\n        padding-left: 3em;\n    }\n",
+      "    .ql-indent-2 {\n        padding-left: 6em;\n    }\n",
+      "    .ql-indent-3 {\n        padding-left: 9em;\n    }\n",
+      "    .ql-indent-4 {\n        padding-left: 12em;\n    }\n",
+      "    .ql-indent-5 {\n        padding-left: 15em;\n    }\n",
+      "    .note {\n        margin: 5px 0px;\n    }\n",
+      "    </style>",
       "</head>",
       "<body>\n",
+      "    <h1 contenteditable=\"\" spellcheck=\"false\">Results</h1>\n",
+      "    <p style=\"text-align:left;padding:0px 0px 0px 0px;\"></p>\n",
       "</body>",
       "</html>")
 }
 
+# creates the text output for the manifest file
 mnfTxt <- function() {
     crrTxt <- paste0(sapply(lstMnf, "[[", 1), ":")
     for (i in seq_along(lstMnf)) {
@@ -369,31 +400,49 @@ mnfTxt <- function() {
     enc2utf8(crrTxt)
 }
 
-add2ZIP <- function(fleZIP = "", crrHdl = NULL, newFle = FALSE, blnZIP = TRUE, txtOut = "") {
-
-    if (!all(class(crrHdl) == c("file", "connection"))) {
-        cat(utils::str(crrHdl))
-        stop("Parameter isn\'t a file handle pointing to a file to be zipped.")
+# adds an input file to the .omv-file (which is a ZIP-archive)
+add2ZIP <- function(fleZIP = "", crrHdl = NULL, crrFle = c(), txtOut = "", ptbOut = NULL, incZIP = TRUE) {
+    if ((!is.character(fleZIP) || length(fleZIP) < 1 || !nzchar(fleZIP)) ||
+        ((is.null(crrHdl) || length(crrHdl) < 1) && (!is.character(crrFle) || length(crrFle) < 1 || !all(nzchar(crrFle))))) {
+        clsRmv()
+        stop("fleZIP (a character with a file name), and either crrHdl (with a connection) or crrFle (with a file name and [optionally] a writing mode) need to be given as arguments.")
     }
 
-    if (all(nchar(txtOut) > 0)) {
-        # writing the manifest requires a bit of special handling because of \r\n on Windows vs. \n on Mac / Linux
-        # thanks to MAgojam for figuring out a solution
-        # for the other files (metadata.json, xdata.json, index.html), it doesn't matter, therefore sep = "\n" is added for all
-        writeLines(txtOut, crrHdl, sep = "\n")
-    }
-
-    crrFle <- summary(crrHdl)[["description"]]
-    close(crrHdl)
-
-    if (blnZIP) {
-        if (newFle) {
-            zip::zip(fleZIP,        basename(crrFle), root = dirname(crrFle))
-        } else {
-            zip::zip_append(fleZIP, basename(crrFle), root = dirname(crrFle))
+    # if a file handle is given, determine the file name, close the handle and remove it (from the calling environment)
+    if (!is.null(crrHdl)) {
+        if (!all(class(crrHdl) == c("file", "connection"))) {
+            clsRmv()
+            stop(sprintf("Parameter isn\'t a file handle pointing to a file to be zipped:\n%s", trimws(utils::capture.output(utils::str(crrHdl)))))
         }
+        crrFle <- rmvTmp(summary(crrHdl)[["description"]])
+        close(crrHdl)
+        rm(list = deparse(substitute(crrHdl)), envir = sys.frame(-1))
+    # if a file name is given, open a connection, write whatever is required
+    } else if (is.character(crrFle)) {
+        crrFle[1] <- rmvTmp(crrFle[1])
+        if (dirname(crrFle[1]) != "." && !dir.exists(file.path(tempdir(), dirname(crrFle[1])))) dir.create(file.path(tempdir(), dirname(crrFle[1])))
+        crrHdl <- file(file.path(tempdir(), crrFle[1]), open = ifelse(length(crrFle) > 1, crrFle[2], "w"))
+        if        (all(nzchar(txtOut))) {
+            # writing the manifest requires a bit of special handling because of \r\n on Windows vs. \n on Mac / Linux
+            # for the other files (metadata.json, xdata.json, index.html), it doesn't matter, therefore sep = "\n" is added for all
+            writeLines(txtOut, crrHdl, sep = "\n")
+        } else if (!is.null(ptbOut)) {
+            ptbOut$serialize(crrHdl)
+        }
+        close(crrHdl)
     }
 
-    unlink(crrFle)
-    rm(crrFle)
+    if (incZIP) {
+        crrPrm <- list(zipfile = fleZIP, files = crrFle[1], root = tempdir())
+        if (file.exists(fleZIP)) do.call(zip::zip_append, crrPrm) else do.call(zip::zip, crrPrm)
+    }
+
+    if (dirname(crrFle[1]) == ".") unlink(file.path(tempdir(), crrFle[1])) else unlink(file.path(tempdir(), dirname(crrFle[1])), recursive = TRUE)
+    rm(crrFle, crrHdl)
+
+    return(TRUE)
+}
+
+rmvTmp <- function(fleNme = "") {
+    return(sub("^/|^\\\\", "", sub(tempdir(), "", fleNme, fixed = TRUE)))
 }
